import { FetchFunction } from '@pnpm/fetcher-base';
import { DirectoryResolution, LocalPackages, Resolution, ResolveFunction, WantedDependency } from '@pnpm/resolver-base';
import { PackageJson, PackageManifest, StoreIndex } from '@pnpm/types';
import { LoggedPkg } from './loggers';
export interface PackageFilesResponse {
    fromStore: boolean;
    filenames: string[];
}
export declare type PackageResponse = {
    body: {
        isLocal: true;
        resolution: DirectoryResolution;
        manifest: PackageManifest;
        id: string;
        normalizedPref?: string;
        updated: boolean;
        resolvedVia?: string;
    };
} | ({
    fetchingFiles?: Promise<PackageFilesResponse>;
    finishing?: Promise<void>;
    body: {
        isLocal: false;
        inStoreLocation: string;
        cacheByEngine: Map<string, string>;
        id: string;
        resolution: Resolution;
        latest?: string;
        normalizedPref?: string;
        updated: boolean;
        resolvedVia?: string;
    };
} & ({
    fetchingRawManifest: Promise<PackageJson>;
} | {
    body: {
        manifest: PackageManifest;
        updated: boolean;
    };
}));
export interface WantedDependency {
    alias?: string;
    pref: string;
}
export interface RequestPackageOptions {
    defaultTag?: string;
    skipFetch?: boolean;
    downloadPriority: number;
    loggedPkg: LoggedPkg;
    currentPkgId?: string;
    prefix: string;
    registry: string;
    shrinkwrapResolution?: Resolution;
    update?: boolean;
    verifyStoreIntegrity: boolean;
    preferredVersions: {
        [packageName: string]: {
            selector: string;
            type: 'version' | 'range' | 'tag';
        };
    };
    localPackages?: LocalPackages;
    sideEffectsCache?: boolean;
}
export declare type RequestPackageFunction = (wantedDependency: WantedDependency, options: RequestPackageOptions) => Promise<PackageResponse>;
export interface FetchPackageToStoreOptions {
    fetchRawManifest?: boolean;
    force: boolean;
    pkgName?: string;
    pkgId: string;
    prefix: string;
    resolution: Resolution;
    verifyStoreIntegrity: boolean;
}
export declare type FetchPackageToStoreFunction = (opts: FetchPackageToStoreOptions) => {
    fetchingFiles: Promise<PackageFilesResponse>;
    fetchingRawManifest?: Promise<PackageJson>;
    finishing: Promise<void>;
    inStoreLocation: string;
};
export default function (resolve: ResolveFunction, fetchers: {
    [type: string]: FetchFunction;
}, opts: {
    networkConcurrency?: number;
    storePath: string;
    storeIndex: StoreIndex;
}): RequestPackageFunction & {
    fetchPackageToStore: FetchPackageToStoreFunction;
    requestPackage: RequestPackageFunction;
};
export declare function getCacheByEngine(storePath: string, id: string): Promise<Map<string, string>>;
